#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" pysymbolscanner
  Copyright 2020 Slash Gordon
  Use of this source code is governed by an MIT-style license that
  can be found in the LICENSE file.
"""
import sys
from pysymbolscanner.command_line import symbolscanner_app

if __name__ == '__main__':
    sys.exit(symbolscanner_app())
